// **************************************************************************************
// el Comandante                                                    guerilla.bplaced.net
// --------------------------------------------------------------------------------------
// Information:
// VSPI CD-Card Breakout Board, Custom Pins, Test Software
// Write and Read Test, List SD-Card Type, SD-Card Size, Directories and Files
// Caution: Complete directory listing takes time, 90 seconds for 1000 entries!
// --------------------------------------------------------------------------------------
// Used Libraries:
// TFT_eSPI --> https://github.com/Bodmer/TFT_eSPI
// **************************************************************************************

#include <SPI.h>
#include <FS.h>
#include <SD.h>
#include <TFT_eSPI.h>                                     // TFT_espi library
TFT_eSPI tft = TFT_eSPI();

// ######################################################################################
// Change values in this area to match your envoirement and desires
// --------------------------------------------------------------------------------------
// Custom VSPI Pins for the SD-Card breakout board - must match the hardware
#define SCK   27
#define MISO  32
#define MOSI  33
#define CS    2
// Delay in ms for messages
uint16_t  messageDelay = 5000;
// ######################################################################################

// Vars
uint32_t  z;
uint32_t  vspiBusFrequency = 2000000;                     // >= 4000000 sd-card mount could fail
uint8_t   cardType;
uint64_t  cardSize;
File      root;
String    szWriteReadPath = "/f4yr.txt";
String    szWriteReadContent = "Fight for your right";

// Create a new SPI class on HSPI or VSPI 
SPIClass vspi = SPIClass(VSPI);

// Setup
void setup()
{
  pinMode(CS, OUTPUT);                                    // set up slave select pin CS as output
  Serial.begin(115200);                                   // serial monitor

  // Init the SPI communication protocol and the SD-Card before TFT_eSPI, so the chip select is defined
  z = 0;
  vspi.begin(SCK, MISO, MOSI, CS);
  if (!SD.begin(CS, vspi, vspiBusFrequency))
  {z = 1;}

  tft.init();                                             // TFT_eSPI
  digitalWrite(TFT_BL, 0);                                // switch backlight off
  tft.setRotation(0);                                     // set the desired rotation
  tft.fillScreen(0x0000);
  tft.setTextColor(0xFFFE, 0x0000);
  tft.setTextSize(1);
  tft.setCursor(0, 8);
  tft.setTextDatum(TL_DATUM);
  if (tft.width() >= 320 || tft.height() >= 320)          // big screen (i.e. WR32-SWC01 - 480 x 320)
  {tft.setTextFont(2);}
  else if (tft.width() < 320 || tft.height() < 320)       // small screen (i.e. LILYGO TTGO-T-Display - 240 x 135)
  {tft.setTextFont(0);}
  digitalWrite(TFT_BL, 1);                                // switch backlight on

  // SD-Card - check error on begin
  if(z == 1)
  {
    Serial.println("Card Mount Error");
    tft.println("Card Mount Error");
    return;
  }

  // SD-Card - get type
  cardType = SD.cardType();
  if(cardType == CARD_NONE)
  {
    Serial.println("No SD-Card");
    tft.println("No SD-Card");
    return;
  }
  Serial.print("SD-Card Type: ");
  tft.print("SD-Card Type: ");
  if(cardType == CARD_MMC)
  {
    Serial.println("MMC");
    tft.println("MMC");
  }
  else if(cardType == CARD_SD)
  {
    Serial.println("SDSC");
    tft.println("SDSC");
  }
  else if(cardType == CARD_SDHC)
  {
    Serial.println("SDHC");
    tft.println("SDHC");
  }
  else
  {
    Serial.println("UNKNOWN");
    tft.println("UNKNOWN");
  }

  // SD-Card - get size
  cardSize = SD.cardSize() / (1024 * 1024);
  Serial.printf("SD-Card Size: %lluMB\n\n", cardSize);
  tft.printf("SD-Card Size: %lluMB\n\n", cardSize);

  // SD-Card - write read
  writereadFile();
  delay(messageDelay);
  tft.fillScreen(0x0000);
  tft.setCursor(0, 8);

  // SD-Card - list content
  root = SD.open("/");
  Serial.println("SD-Card Directories:");
  Serial.println("--------------------");
  tft.println("SD-Card Directories:");
  tft.println("----------------------");
  listSDCardContent(root);

  // SD-Card - all done
  Serial.println("Done!");
  Serial.println("****************************************");
  tft.println("Done!");
}


// Loop
void loop()
{
// notting to do
}


// Write file and read file test
void writereadFile()
{
  // Write file
  Serial.println("Write file:");
  Serial.println(szWriteReadPath);
  tft.println("Write file:");
  tft.println(szWriteReadPath);
  root = SD.open(szWriteReadPath, FILE_WRITE);
  if(!root)
  {
    Serial.println("Write error");
    tft.println("Write error");
    return;
  }
  if(root.print(szWriteReadContent))
  {
  Serial.println("Write okay:");
  Serial.println(szWriteReadContent);
  Serial.println("");
  tft.println("Write okay:");
  tft.println(szWriteReadContent);
  tft.println("");
  }
  else
  {
  Serial.println("Write error\n");
  tft.println("Write error\n");
  }
  root.close();

  // Read file
  Serial.println("Read file:");
  Serial.println(szWriteReadPath);
  tft.println("Read file:");
  tft.println(szWriteReadPath);

  root = SD.open(szWriteReadPath);
  if(!root)
  {
    Serial.println("Read error");
    tft.println("Read error");
    return;
  }
  Serial.println("Read okay:");
  tft.println("Read okay:");
  while(root.available())
  {
    z = root.read();
    Serial.write(z);
    tft.write(z);
  }
  root.close();
  Serial.println("");
  Serial.println("");
  tft.println("");
  tft.println("");
}


// List SD-Card Content
void listSDCardContent(File dir)
{
  while (true)
  {
    File entry =  dir.openNextFile();
    if (! entry)                                          // no more entry
    {break;}

    if (entry.isDirectory())                              // directory, no size
    {
      Serial.print(entry.name());
      Serial.println("/");
      tft.print(entry.name());
      tft.println("/");
      listSDCardContent(entry);                           // list files in current directory
    }
    else                                                  // file in current directory and file size
    {
      Serial.print(" ");
      Serial.print(entry.name());
      Serial.print(" = ");
      Serial.print(entry.size(), DEC);
      Serial.println(" Byte");
      tft.print(" ");
      tft.print(entry.name());
      tft.print(" = ");
      tft.print(entry.size(), DEC);
      tft.println(" Byte");
    }
    entry.close();
  }
}
